from zope import schema
from zope.interface import Interface


class IRequest(Interface):
    """Provides requests.request implementation"""
    req_kwargs = schema.Dict(
            title=u'kwargs',
            description=u'A dict-like object containing a default set of' +\
                        u'kwargs to be used by request()',
            default={})
    gooble_warnings = schema.Bool(
            title=u'Gooble warnings',
            description=u'True indicates to gooble warnings issued by calls to request()',
            default=False)
    def request(*args,**kwargs):
        """wrapper for requests.request
        
        kwargs delivered to method will over-ride competing entries found in
        IRequest.req_kwargs defaults
        """

class IRequestResolver(Interface):
    def __call__(request=None):
        """Return IRequest provider
        
        Return an appropriate IRequest provider based on the following
        resolution order for the related requests.request object:
         - return kwarg['request'] if it provides IRequest
         - return unnamed utility singleton providing IRequest in ZCA if found
         - return newly created IRequest with new requests.request
        
        kwargs:
            request: IRequest provider
        """


import requests
import warnings
from zope import component
from zope.component.factory import Factory
from zope import interface
from zope.schema.fieldproperty import FieldProperty
from .interfaces import IRequest, IRequestResolver
from sparc.config import IConfigContainer


@interface.implementer(IRequest)
class Request(object):
    req_kwargs = FieldProperty(IRequest['req_kwargs'])
    gooble_warnings = FieldProperty(IRequest['gooble_warnings'])
    
    def __init__(self, **kwargs):
        self.req_kwargs = kwargs.get('req_kwargs', {})
        self.gooble_warnings = kwargs.get('gooble_warnings', False)
        self.requester = kwargs.get('requester', requests)
    
    def request(self, *args, **kwargs):
        kwargs_updated = self.req_kwargs.copy()
        kwargs_updated.update(kwargs)
        if self.gooble_warnings:
            with warnings.catch_warnings():
                warnings.simplefilter("ignore")
                return self.requester.request(*args, **kwargs_updated)
        else:
            return self.requester.request(*args, **kwargs_updated)
requestFactory = Factory(Request)
SparcRequest = requestFactory()

@interface.implementer(IRequestResolver)
def request_resolver(**kwargs):
    """Resolve for a IRequest and return
    
    This will return a IRequest based on the following resolution order:
    1. 'request' is given in kwargs and provides IRequest
    2.  IRequest is available as a unnamed utility in zca
    3.  A new IRequest is created and returned
    """
    if 'request' in kwargs and IRequest.providedBy(kwargs['request']):
        return kwargs['request']
    sm = component.getSiteManager()
    return sm.queryUtility(IRequest, default=requestFactory())

@interface.implementer(IRequest)
@component.adapter(IConfigContainer)
class RequestFromConfigContainer(Request):
    """Adapts from the first RequestOptions found"""
    def __init__(self, context):
        self.context = context
        RequestOptions = context.mapping().get_value('RequestOptions', default={})
        super(RequestFromConfigContainer, self).__init__(**RequestOptions)



import os
import unittest
import zope.testrunner
from zope import component
from sparc.testing.fixture import test_suite_mixin
from zope.component import createObject
from ..testing import SPARC_REQUESTS_INTEGRATION_LAYER
from .. import IRequest, IRequestResolver

import sparc.requests.request

class SparcRequestsRequestTestCase(unittest.TestCase):
    layer = SPARC_REQUESTS_INTEGRATION_LAYER
    sm = component.getSiteManager()
    
    def unregister_request(self):
        self.sm.unregisterUtility(
            component=sparc.requests.request.SparcRequest, 
            provided=IRequest)
    
    def register_request(self):
        self.sm.registerUtility(
            component=sparc.requests.request.SparcRequest, 
            provided=IRequest)

    def test_factory(self):
        self.assertTrue(IRequest.providedBy(createObject(u'sparc.requests.request')))
    
    def test_singleton(self):
        self.assertTrue(IRequest.providedBy(self.sm.getUtility(IRequest)))
    
    def test_request_resolver(self):
        req = createObject(u'sparc.requests.request')
        
        resolver = self.sm.getUtility(IRequestResolver)
        self.assertIs(resolver(request=req), req)
        
        registered_req = self.sm.getUtility(IRequest)
        self.assertIs(resolver(), registered_req)
        
        self.unregister_request()
        self.assertIsNot(resolver(), registered_req)
        
        self.register_request()
        self.assertIs(resolver(), registered_req)
        

class test_suite(test_suite_mixin):
    layer = SPARC_REQUESTS_INTEGRATION_LAYER
    package = 'sparc.requests'
    module = 'request'
    
    def __new__(cls):
        suite = super(test_suite, cls).__new__(cls)
        suite.addTest(unittest.makeSuite(SparcRequestsRequestTestCase))
        return suite


if __name__ == '__main__':
    zope.testrunner.run([
                         '--path', os.path.dirname(__file__),
                         '--tests-pattern', os.path.splitext(
                                                os.path.basename(__file__))[0]
                         ])